	program emst_test

!  Program to test the implementation of the EMST mixing model.
!   Z. Ren & S. B Pope, November 2002

!  Non-physical initial conditions are set for np particles with nc compositions.
!  The evolution of the particles is calculated using EMST for a normalized time omt_end.
!  The output file consists of three columns:
!	1/ the normalized time omt
!	2/ the calculated variance function
!	3/ the expected value of the variance function = exp(-omt)
         use emst_subs, only: emst_rnu
	implicit none
	integer, parameter :: npd=200 ! dimension for particles
	integer, parameter :: ncd=80	  ! dimension for compositions

	integer :: np, nc, info, j, lu=7
      real(kind(1.e0)) :: f(npd,ncd), state(npd), wt(npd), fscale(ncd),
	1  omdt, cvars(6), omdt_min, omdt_max, omt_end, omt, vf0, vf,
	1  u(1)

!------------------- test specification ------------------------------

	np=100	! number of particles
	nc=3	! number of compositions

	omt_end  = 20.	! duration of simulation
	omdt_min = 0.	! smallest time step
	omdt_max = 1.e-2 ! largest time step

	call emst_rnu(fscale(1:nc),nc)
	fscale=fscale+0.1	! set scales randomly, uniform in [0.1, 1.1]

	call emst_rnu(wt(1:np),np) ! set weights randomly, uniform in [0, 1]

	do j=1,nc	! set compositions randomly, uniform in [0, 1]
	   call emst_rnu(f(1:np,j),np)
	end do

         if(nc>1) f(2:np,1)=f(1,1)  !if nc>1, reset f(:,1) to a delta function

	cvars(1:6)=0.	! set contol variables to default settings

!---------------end of test specification ------------------------------

	call var_func(vf0)	! compute the initial variance function
	open( 7, file = 'emst.out' )	! output file

! call to emst (mode=1) to initialize the state (age) variable state(1:np)
	call emst(1,np,nc,f(1:np,1:nc),state,wt,omdt,fscale,cvars,info)
	if( info /= 0 ) then
	    write(0,*)'emst initialization failed'
	    stop
	endif

	omt=0.	!  loop over time steps
	do while( omt < omt_end )
	  call var_func(vf)	! calculate and output variance function
	  write(7,100) omt, vf/vf0, exp(-omt)
100	  format(1p,10e15.6)

!  choose time step randomly,uniform in [omdt_min, omdt_max]
	  call emst_rnu(u,1)
	  omdt=omdt_min + u(1) * (omdt_max-omdt_min)
	  omt = omt + omdt

! perform mixing, mode=2
	  call emst(2,np,nc,f(1:np,1:nc),state,wt,omdt,fscale,cvars,info)

	  if( info /= 0 ) then
	    write(0,*)'emst failed'
	    stop
	  endif
	end do	! end of time-stepping loop

	stop

	contains	!-------------------------------------------------------

	subroutine var_func(vf)
!  Compute the variance function, vf.
!  vf is defined as the trace of the covariance of the scaled compositions.
	real(kind(1.e0)), intent(out) :: vf
	real(kind(1.e0)) :: sumwt, mean(nc), var(nc),gg(np)
	integer :: j,kk

! check that scales are strictly positive
	if( minval(fscale(1:nc)) <= 0. ) then
           write(0,*)'emst_test: non-positive fscale in var_func'
	   stop
	endif

! check that weights are non-negative
        if( minval(wt(1:np)) <= 0. ) then
           write(0,*)'emst_test: negative weight in var_func'
	   stop
	endif

! form means and variances (of un-scaled compositions)
	sumwt   = sum(wt(1:np))
        mean(1:nc)=0
       var(1:nc)=0
      do j=1,nc
        do kk=1,np
           mean(j)=mean(j)+wt(kk)*f(kk,j)
        enddo
        mean(j)=mean(j)/sumwt
        do kk=1,np
           var(j)=var(j)+wt(kk)*(f(kk,j)-mean(j))**2
        enddo
       var(j)=var(j)/sumwt
!ren           mean(j) = dot_product( wt(1:np), f(1:np,j) )/sumwt
!ren	   var(j) = dot_product( wt(1:np), (f(1:np,j)-mean(j))**2 )/sumwt
      enddo

! scale variances
	vf = sum( var(1:nc)/fscale(1:nc)**2 )

	end subroutine var_func

	end program emst_test
